%This program performs ICA on a observed data using the Beta-Divergence.
%Neil Gadhok
%v1.0 Feb 2, 2005
%

function [Wfinal,ufinal,Wss] = BetaD_ICA(x,W,u,B,stopping_criterion,maxiteration_BFGS,maxiteration_linesearch,p_i)

%Determine the number of source vectors.
%Determine m and n
    %x must be a row vector
    %m is the number of sources
    %n is t%Behe number of samples per source
[m,n]= size(x);
W1 = W;
u1 = u;

if B == 0,
    v1 = zeros(m^2,1);
    v  = formv2(W); %v is a column vector
    i = 1;
    Wss(:,:,i) = W;
       
    %BFGS method to find W
        G = diag(ones(1,m^2));    %G is an estimate of the inverse Hessian.
        while (norm(v1-v) > stopping_criterion) & (i < maxiteration_BFGS)
            
            if i==1
            else
                v = v1;           %Update v
                W= deformv2(v);   %Update W and u to calculate the new direction.
            end
            
            
            p = -G*(BetaD_Deriv(x,W,u,B,p_i));
            alpha = 1;  %alpha is a line search. We have set it to 1, but it should be a function.

            %Line search
                sigma = 1.e-4;    % minimum decrease in f
                alphamin = 1.e-4; % minimum step length
                pgx = p'*BetaD_Deriv(x,W,u,B,p_i);
                v1 = v + alpha*p;  
                W2 = deformv2(v1);
                u2 = u;
                fxn = BetaD_Lbeta(x,W2,u2,B,p_i);
                fx = BetaD_Lbeta(x,W,u,B,p_i);
                i_linesearch = 1;
                
                Temp3(i) = 0;
                
                while( fxn > fx + sigma*alpha*pgx) & (alpha > alphamin) & (i_linesearch < maxiteration_linesearch); %We must minimize
                    mu = -0.5 * pgx * alpha / (fxn - fx - alpha * pgx );
                    if mu < .01,
                        mu = .5; %don't trust quadratic interpolation from far away
                    end
                    alpha = mu * alpha;
                    v1 = v + alpha*p;  
                    W2 = deformv2(v1);
                    fxn = BetaD_Lbeta(x,W2,u2,B,p_i);
                    i_linesearch = 1 + i_linesearch;
                    Temp3(i) = 1;
                end
            %end Line search

            v1 = v + alpha*p;     %Now we have the a proper alpha
            W = deformv2(v1);       % v1
            W1= deformv2(v);        % v
            u1 = u;
            s = alpha*p;
            y  = BetaD_Deriv(x,W,u,B,p_i) - BetaD_Deriv(x,W1,u1,B,p_i);
            I = diag(ones(1,m^2));
            G =  (I - (s*(y'))/((y')*s))*G*(I - (y*(s'))/((y')*s)) + (s*(s'))/((y)'*s);
            i = i+1;
            
            Wss(:,:,i) = W;
            
        end
    
    Wfinal = deformv2(v1);
    ufinal = mean(x,2);
    
    
else
    v1 = zeros(m^2+m,1);
    v  = formv(W,u); %v is a column vector
    i = 1;
    Wss(:,:,i) = W;
    %BFGS method to find W and u
        G = diag(ones(1,m^2+m));    %G is an estimate of the inverse Hessian.
        while (norm(v1-v) > stopping_criterion) & (i < maxiteration_BFGS) | (i == 1)
  
            if i==1
            else
                v = v1;             %Update v
                [W,u] = deformv(v); %Update W and u to calculate the new direction.
            end
            
       
            p = -G*(BetaD_Deriv(x,W,u,B,p_i)); %Calculate new direction
            alpha = 1;  %alpha is a line search. We have set it to 1, but it should be a function.
    
          %Line search
              sigma = 1.e-4; % minimum decrease in f
              alphamin = 1.e-4; % minimum step length
              pgx = p'*BetaD_Deriv(x,W,u,B,p_i);
              v1 = v + alpha*p;  
              [W2,u2] = deformv(v1); 
              fxn = BetaD_Lbeta(x,W2,u2,B,p_i);
              fx = BetaD_Lbeta(x,W,u,B,p_i);
              i_linesearch = 1;
              
              Temp3(i) = 0;
              
              while( fxn > fx + sigma*alpha*pgx) & (alpha > alphamin) & (i_linesearch < maxiteration_linesearch); %We must minimize
                  mu = -0.5 * pgx * alpha / (fxn - fx - alpha * pgx );
                  if mu < .01,
                      mu = .5; %don't trust quadratic interpolation from far away
                  end
                  alpha = mu * alpha;
                  v1 = v + alpha*p;  
                  [W2,u2] = deformv(v1); 
                  fxn = BetaD_Lbeta(x,W2,u2,B,p_i);
                  i_linesearch = 1 + i_linesearch;
                  Temp3(i) = 1;
              end 
              if (fxn > fx + sigma*alpha*pgx)
                  %Line search failed. Thus, the BFGS approx of the Hessian has become corrupt, or we are near the minimum (flat valley) and the step size is too big. So restart.
                  G = diag(ones(1,m^2+m));
                  p = -G*(BetaD_Deriv(x,W,u,B,p_i));
                  alpha = 1;
              end
              
          %End Line search. Now we have the best alpha
    
            v1 = v + alpha*p; %Update v1
            [W,u] = deformv(v1);   % v1
            [W1,u1] = deformv(v);  % v
            s = alpha*p;
            y  = BetaD_Deriv(x,W,u,B,p_i) - BetaD_Deriv(x,W1,u1,B,p_i);
            I = diag(ones(1,m^2+m));
            G =  (I - (s*(y'))/((y')*s))*G*(I - (y*(s'))/((y')*s)) + (s*(s'))/((y)'*s);
                   
            Wss(:,:,i) = W/abs(norm(W));
            
            i = i+1;
       end
       
       [Wfinal,ufinal] = deformv(v1);
   end

