%This experiment studies the rotation sensitivity of ICA to outliers.
%Experiment 1: Rotation sensitivity analysis for a mixture of a uniform (Source 1) and Uniform (Source 2) mixture from the exponential power distribution.
%Must change the save directories each time. 
%No numerical whitening. NOTE: MUST HAVE RUN CCECE05_EXP1 to generate the CCECE05_Dataset.
%Neil Gadhok, Mar 6, 2005, v1.0

%NOTE: Boxplot.m modified with the following:

%if ylims(1)==ylims(2);
%    delta = .5;
%    ylims = [(ymin-delta) (ymax+delta)];
%end

%Variable breakdown:
%
%Cntrsts(A,B,C,D,E,F,G,H)
%Contrsts(1:10,        1:2 [100,200],    1:2 [0,45],  1:2 [0,1],    1:5 [0,1,3,5,7], 1 [0],     1:17,                1:50)
%Contrsts(NumTestRuns, NumSamples,       Rotation,    NumOutliers,  Outliers1,        Outliers2,Cntrst_FastICA_G1,1
%                                                                                               Cntrst_FastICA_G2,2
%                                                                                               Cntrst_FastICA_G3,3
%                                                                                               Cntrst_FastICA_G4,4
%                                                                                               API_FastICA,      5
%                                                                                               Cntrst_Infomax,   6
%                                                                                               API_Infomax,      7
%                                                                                               Cntrst_JADE,      8
%                                                                                               API_JADE,         9
%                                                                                               Cntrst_Radical,   10
%                                                                                               API_Radical,      11
%                                                                                               Cntrst_BetaD_0,   12
%                                                                                               Cntrst_BetaD_05,  13
%                                                                                               Cntrst_BetaD_1,   14
%                                                                                               Cntrst_BetaD_2,   15
%                                                                                               Cntrst_BetaD_3,   16
%                                                                                               API_BetaD,        17   


%Note: When NumOutliers = 1 [So, zero outliers), the only values solved for, is when Outliers1 = 1 [0] and Outliers2 = 1[0]

    
%AmariPerf(A,B,C,D,E,F,G,H)
%AmariPerf(1:10,        1:2 [100,200],    1:2 [0,45],  1:2 [0,1],    1:5 [0,1,3,5,7], 1 [0],   1:12,                1)
%AmariPerf(NumTestRuns, NumSamples,       Rotation,    NumOutliers,  Outliers,        Outliers,Cntrst_FastICA_G1,1
%                                                                                              Cntrst_FastICA_G2,2
%                                                                                              Cntrst_FastICA_G3,3
%                                                                                              Cntrst_FastICA_G4,4
%                                                                                              Cntrst_Infomax,   5
%                                                                                              Cntrst_JADE,      6
%                                                                                              Cntrst_Radical,   7
%                                                                                              Cntrst_BetaD_0,   8
%                                                                                              Cntrst_BetaD_05,  9
%                                                                                              Cntrst_BetaD_1,   10
%                                                                                              Cntrst_BetaD_2,   11
%                                                                                              Cntrst_BetaD_3,   12

%Note: When NumOutliers = 1 (So, zero outliers), the only values solved for, is when Outliers1 = 1 [0] and Outliers2 = 1[0]


%Matlab Setup
clear all;
close all;
clc;

%Variable setup
NumSamples = [100,200];  %Number of source samples
NumOutliers = [0,1]; %Number of outliers to introduce at the sensor
Rotation = [0,45]; %The different rotation matrices used to mix the sources.
Outliers = [0,1,3,5,7]; %The outlier is at 0,1,3,5, or 7 std deviations from the mean (Assuming symmteric, zero mean, unit variance distributions
NumTestRuns = 10;

NumRandomNumbers = NumSamples(end)*NumTestRuns;
m = 2; %The number of sources

%Generate Random Numbers for experiment and save them
%DataSet_Uniform1 = unifrnd(-sqrt(3),sqrt(3),1,NumRandomNumbers); %0 mean, unit variance. mean = 1/2(a+b), variance = 1/12(b-a)^2, kurt = -1.2
%DataSet_Uniform2 = unifrnd(-sqrt(3),sqrt(3),1,NumRandomNumbers); %0 mean, unit variance. mean = 1/2(a+b), variance = 1/12(b-a)^2, kurt = -1.2
%mkdir('e:\Thesis\Results\',date);
mkdir(['e:\Thesis\Results\',date],'Exp2');
load 'e:\Thesis\Results\11-Apr-2005\Exp1\CCECE05_DataSet.mat'  %The same dataset as in Experiment 1


%Breakup the Datasets into NumTestRuns and repeat
for i = 1:NumTestRuns
    %Break up into number of samples (100,200 or 500)
    display(['TestRun ',num2str(i),' of ',num2str(NumTestRuns)])
     
    for j = 1:length(NumSamples)
        
        N = NumSamples(j); %N is the number of samples.
        display(['Number of Samples = ',num2str(N)])
                
        %Create the sources
          %[1 + (i-1)*(NumSamples(j)), (i)*(NumSamples(j))]
         Source1 = DataSet_Uniform1(1 + (i-1)*(NumSamples(j)):(i)*(NumSamples(j))) ;
         Source2 = DataSet_Uniform2(1 + (i-1)*(NumSamples(j)):(i)*(NumSamples(j))) ;
         s = [Source1;Source2]; %The source vector is m x N
         
        %Select the rotation matrix for mixing.
         for k = 1:length(Rotation)
             RotationMixingMatrix = [ cos(Rotation(k)) -sin(Rotation(k)) ; sin(Rotation(k)) cos(Rotation(k)) ]; % RotationMixingMatrix  = A
             A = RotationMixingMatrix;
             display(['Rotation Angle = ',num2str(Rotation(k))])
             
             %Mix the sources and obtain our mixed sources without outliers
             sM = A*s;
             
             %Whiten the mixed sources without outliers to introduce numerical whitening errors
              %sW = sM - mean(sM,2)*ones(1,length(sM)); %Make sM zero mean
              %[E, D] = eig (cov(sW', 1)); %The 1 normalizes by N.
              %whiteningMatrix_sW = inv (sqrt (D)) * E';
              %sW = whiteningMatrix_sW*sW;
              
              sW = sM;
              whiteningMatrix_sW = eye(2);
              
              
             %Introduce outliers at the sensor (Either no outliers, 1 or 2 for one or both sources, we must maintain the same number of outliers, but those outlier locations may change
             for i_0 = 1:length(NumOutliers)
                 
                 if (NumOutliers(i_0) == 0) %No Outliers
                     Source1Outliers = [];
                     Source2Outliers = [];
                     j_0 = 1;
                     k_0 = 1;
                     
                     %Obtain our observations at the sensor
                     x = [sW(1,:),Source1Outliers; sW(2,:),Source2Outliers];
                     %Cntrst Call
                     Cntrsts(i,j,k,i_0,j_0,k_0,:,:) =  CCECE05_Exp2_Cntrst_caller(x,A,whiteningMatrix_sW);
                     %ICA Call
                     AmariPerf(i,j,k,i_0,j_0,k_0,:,:) = CCECE05_Exp2_ICA_caller(x,A,whiteningMatrix_sW);
                     
                 else
                     for j_0 = 1:length(Outliers) %for source 1
                             k_0 = 1; %MUST FIX IF WE CHANGE THE NUMBER OF OUTLIERS
                             %Obtain the outliers for source 1 and 2
                             Source1Outliers = Outliers(j_0);
                             Source2Outliers = 0;
                             
                             %Obtain our observations at the sensor
                             x = [sW(1,:),Source1Outliers; sW(2,:),Source2Outliers];
                            
                             %Cntrst Call
                             Cntrsts(i,j,k,i_0,j_0,k_0,:,:) =  CCECE05_Exp2_Cntrst_Caller(x,A,whiteningMatrix_sW);
                                               
                             %ICA Call
                             AmariPerf(i,j,k,i_0,j_0,k_0,:,:) = CCECE05_Exp2_ICA_Caller(x,A,whiteningMatrix_sW);
                             
                     end % end 1:length(Outliers) for source 1
                 end % (NumOutliers(i_0) == 0) %No Outliers
                 
             end % end 1:length(NumOutliers)
             
         end %end 1:length(Rotation)
                  
    end %end 1:length(NumSamples)
        
end %end 1:NumTestRuns

save(['e:\Thesis\Results\',date,'\Exp2\CCECE05_Cntrsts'],'Cntrsts');
save(['e:\Thesis\Results\',date,'\Exp2\CCECE05_AmariPerf'],'AmariPerf');
         

