%This file calculates the FastICA objective function landscape.
%Neil Gadhok, v1.0, Dec 4, 2004.
function [J1] = FastICA_Cntrst(x,W,v,Gf)
%We want to maximize the negentropy. However, for consistancy, we will output
%-J1, thus, we want to minimize -J1 (or maximize the negentropy).


%FastICA uses an approximation to negentropy as a non-Gaussianity measure.  
%Negentropy is a nonnegative measure and is zero for a Gaussian variable. Thus,
%the objective is to maximize the negentropy to find the most non-Gaussian source
%and thus, an independent source. The FastICA program MAXIMIZES an approximation of
%negentropy (Eq. 8.25, HyKO01) by a fixed point iteration. Here we are calculating
%optimization surface (negentropy) which FastICA tries to find a maximum. There are
%a number of constraints that the algorithm imposes, unit norm demixing matrix, and 
%orthogonal demixing matrix rows.

[m,n] = size(x);

  
       
    %Create W's
    J1 = 0;

        %Eq 8.25
        %J(y) = [E{G(y)} - E{G(v)}]^2, where y = w'x, w is a row of the demixing matrix W,
        %x is the observation matrix that has been whitened and is zero mean, and v is a 
        %Gaussian random variable of zero mean and unit variance. G is a nonquadratic 
        %equation that when chosen wisely gives an approximation to negentropy. We have
        %an objective function for each row of W.
        for i = 1:m,
            y = W(i,:)*x;
            
            %y should be zero mean and have unit variance
            y = y - mean(y,2)*ones(1,length(y)); %Make v zero mean
            [E, D] = eig (cov(y', 1)); %The 1 normalizes by N.
            whiteningMatrix_y = inv (sqrt (D)) * E';
            y = whiteningMatrix_y*y;
                
            J1 = J1 + ((1/n)*sum(Gfunctions(y,Gf)) - (1/n)*sum(Gfunctions(v,Gf))).^2 ;  %Eqn 8.25
            %J1 = J1 + (1/12)*((1/n)*sum(Gfunctions(y,4))).^2 + (1/48)*(kurtosis(y,0).^2) ; %Eqn 8.23

        end
  
      J1 = -J1; %We want to maximize negentropy, but for consistency we will minimize -J1.
!